# infra-pipeline-images

The Infra pipeline images project was created to shrink the size of the orignal automation-image. 
Which had grown to 2.41 GB, and was used in most stages of the infra pipelines.

This project contains a build system to create and push dedicated images used for the various stages of the infra pipeline.

[[_TOC_]]



## Local development and testing

You can build these containers locally to test with the following script.

```bash 
./local_build.sh
```

It will build containers locally in your docker engine with using just the short name for the image and tag is called local. This will still have identical content to the released containers. The first time you run this it may be a little slow, but with subsequent builds the caching should make testing changes very fast.
Its safe to run this straight away, it won't interfere with any released components 

### Local Testing


The container image tests have been written using Googles container-structure-test tool - https://github.com/GoogleContainerTools/container-structure-test

The local_build.sh script will run the tests after build if it finds a file in the image directory called tests.yaml. You can also run an individual test for a single image by setting up as below. This is handy when you want to expand the tests and just run for a single image repeatedly.

```bash
# Test the locally built notify image
container-structure-test test --image  notify:local --config images/notify/tests.yaml
```


## Dev pipelines

To create a development container image start a new branch. The branch name will become the container tag and each commit to the branch will overwrite the container image tag with the branch name.
One this tag is created you can reference this image from other pipelines (deploy-templates etc) by using the image name and the branch name as the tag. 


The development containers are stored in this repositories container registry. 
You can browse the available images and tags etc here: [Dev Containers](https://gitlab.builder.ai/devsecops/build-images/infra-pipeline-images/container_registry). These are suitable for testing on feature branches of deploy-templates etc or can even be pulled locally for testing.


## Release Pipelines

Currently, the release process is to manually create a new tag after the feature branch has merged to master. The merge to master will NOT trigger a pipeline. 
This must be done by the creation of a tag. The new tag creation will trigger a pipeline, and this will create all the images at that tag version and push them to this projects container registry with the new tag.
[Infra Containers](https://gitlab.builder.ai/devsecops/build-images/infra-pipeline-images/container_registry). 
Once built the tag will appear and can be used from other pipelines etc.

# Guides

## Adding a new container

1. Create a new folder for the image in the images directory. This folder name is important it will form the main part of the container name. 
2. Create its Dockerfile in the new directory
3. Modify the .gitlab-ci.yml file, add in a new job extending the build-template and align it to the new folder you created
4. Update the Dockerfile and test locally.
5. Once you are happy, from a feature branch push the code and check the Pipeline can also build.
6. When its ready for release - raise an MR. Once merged create a new tag.

## Changing versions of software

See the [versions.source](./versions.source) file. Update this file in a branch. Check the containers all build. Test these new images by ensuring they pass the local tests and still perform their tasks in their other pipelines.
Finally raise an MR. Once merged create a new tag.

## Triggering the pull tests job

If you have Gitlab CLI installed you can trigger a pull test pipeline with the command

glab ci run -b master --variables RUN_PULL_TESTS:true --variables SKIP_BUILD:true

If you trigger this from the console, start a pipeline from the required branch, and the set the following CI Vars

```text
RUN_PULL_TESTS=true  # Run the pull tests jobs
SKIP_BUILD=true      # Skip the stage which builds all the images
```

### Pull Tests Dashboard

https://monitor.svc.builder.ai/d/fb1afaa1-f094-49dc-9cea-3cfbf93b02e7/gitlab-ci-pipeline-image-pull-times?orgId=1

## Example Pull times

#### Original image (tested locally)

```text
$ time docker pull registry.gitlab.builder.ai/shared-services/docker-images/automation-image:0.0.72

0.0.72: Pulling from shared-services/docker-images/automation-image
0fb668748fc8: Pull complete 
a50fd0943d82: Pull complete 
ea31f72c2100: Pull complete 
467307b58e32: Pull complete 
Digest: sha256:1360f4686896f141fc10eef0bfaf0fb019c9c28988e73d336628e625d6096e5d
Status: Downloaded newer image for registry.gitlab.builder.ai/shared-services/docker-images/automation-image:0.0.72
registry.gitlab.builder.ai/shared-services/docker-images/automation-image:0.0.72

real	0m35.054s
user	0m0.020s
sys	0m0.007s
```

#### New preflight-checks image

```text
$ time docker pull registry.gitlab.builder.ai/devsecops/build-images/infra-pipeline-images/preflight-checks:0.0.2 
0.0.2: Pulling from devsecops/build-images/infra-pipeline-images/preflight-checks
707e32e9fc56: Already exists 
04ef5072fab2: Pull complete 
814eb1a8303a: Pull complete 
b03f5dbf1908: Pull complete 
946503b8aa7f: Pull complete 
c8640a2f9c43: Pull complete 
52b9cb497584: Pull complete 
55e43240850d: Pull complete 
Digest: sha256:186eb32088971055457033aae76591ce2900d9699f6ab89ea1a757b0578a8495
Status: Downloaded newer image for registry.gitlab.builder.ai/devsecops/build-images/infra-pipeline-images/preflight-checks:0.0.2
registry.gitlab.builder.ai/devsecops/build-images/infra-pipeline-images/preflight-checks:0.0.2

real	0m5.370s
user	0m0.000s
sys	0m0.011s

```

## Todo

* Container scanning would be nice
* It would be nice to further trim the AZ CLI, which comes in at a big fat 1GB just to install...
* We can update the AZ CLI further, but MS has changed some of the casing on the DNS Record results. Thanks MS
* We might also be able to shrink the create cluster further by moving away from AZ CLi and using the SDK directly from python (While we are still living with platform-automation-tools)
* These are being built by docker-in-docker, there are better ways to create these
* They are currently stored in the local Container registry - we may want to push to the central one.
* The gitlab rules will build all containers for each new tag or branch. We may want to be more selective later

