#!/bin/bash
#set -x
set -e

LOCAL_TAG="local" # TODO << Something better than this tag

export CI_JOB_TOKEN=$(cat ~/.gitlab-pat)  # Containers which install platform-automation-libs inject a token :'(

bold=$(tput bold)
normal=$(tput sgr0)

# Check test binary exists


if which container-structure-test; then
  echo "Found test binary on path."
else
  echo -e "${bold}Did not find container-structure-test${normal}\n you may need to install with instructions from https://github.com/GoogleContainerTools/container-structure-test and ensure its on the path."
  exit 1
fi


# Source the current versions
for dir in images/*/; do
    echo -e "\n${bold}Processing directory:${normal} $dir\n"
    CONTAINER_IMAGE=$(basename "$dir")
    DOCKERFILE="${dir}Dockerfile"
    BUILD_ARGS=""
    echo -e "\n${bold}Dockerfile is:${normal} $DOCKERFILE\n"

     # Collect the required build-args by grepping the Dockerfile
    while read -r line; do
        if [ -z "$line" ]; then continue; fi
        BUILD_ARGS="$BUILD_ARGS --build-arg $line"
        echo "After appending: $BUILD_ARGS"
    done <<EOF
    $(grep ^ARG "$DOCKERFILE" | sed 's/ARG //g')
EOF
    # chomp the leading space.
    BUILD_ARGS="${BUILD_ARGS# }"
    echo "${bold}Build args are:${normal} $BUILD_ARGS"

    # Run the docker build stage
    source versions.source
    docker build -t "${CONTAINER_IMAGE}:${LOCAL_TAG}" -f "$DOCKERFILE" ${BUILD_ARGS} .

    # Run the test stage
    if [[ -e "$dir/tests.yaml" ]]; then
      echo -e "\n${bold}Running tests:${normal} $dir/tests.yaml\n"
      container-structure-test test --image "${CONTAINER_IMAGE}:${LOCAL_TAG}" --config "${dir}tests.yaml"
    else
      echo -e "\n${bold}No Tests found for image: ${CONTAINER_IMAGE}:${LOCAL_TAG} ${normal}\n"
    fi

done
